@@ -186,6 +186,33 @@ The data file is left at the end of testing so that it is possible to use normal
 examine it.
 
 
+Coverage Config File
+--------------------
+
+This plugin provides a clean minimal set of command line options that are added to pytest.  For
+further control of coverage use a coverage config file.
+
+For example if tests are contained within the directory tree being measured the tests may be
+excluded if desired by using a .coveragerc file with the omit option set::
+
+    py.test --cov-config .coveragerc
+            --cov myproj
+            myproj/tests/
+
+Where the .coveragerc file contains file globs::
+
+    [run]
+    omit = tests/*
+
+For full details refer to the `coverage config file`_ documentation.
+
+.. _`coverage config file`: http://nedbatchelder.com/code/coverage/config.html
+
+Note that this plugin controls some options and setting the option in the config file will have no
+effect.  These include specifying source to be measured (source option) and all data file handling
+(data_file and parallel options).
+
+
 Limitations
 -----------
 
@@ -217,6 +244,7 @@ Michael Foord for unittest2.
 No doubt others have contributed to these tools as well.
 """
 
+
 def pytest_addoption(parser):
     """Add options to control coverage."""
 
@@ -237,7 +265,7 @@ def pytest_configure(config):
     """Activate coverage plugin if appropriate."""
 
     if config.getvalue('cov_source'):
-        config.pluginmanager.register(CovPlugin(config), '_cov')
+        config.pluginmanager.register(CovPlugin(), '_cov')
 
 
 class CovPlugin(object):
@@ -248,7 +276,7 @@ class CovPlugin(object):
     distributed slave.
     """
 
-    def __init__(self, config):
+    def __init__(self):
         """Creates a coverage pytest plugin.
 
         We read the rc file that coverage uses to get the data file
@@ -268,16 +296,26 @@ class CovPlugin(object):
         cov_report = session.config.getvalue('cov_report') or ['term']
         cov_config = session.config.getvalue('cov_config')
 
-        name_to_cls = dict(Session=cov_core.Central,
-                           DSession=cov_core.DistMaster,
-                           SlaveSession=cov_core.DistSlave)
         session_name = session.__class__.__name__
-        controller_cls = name_to_cls.get(session_name, cov_core.Central)
+        is_master = (session.config.pluginmanager.hasplugin('dsession') or
+                     session_name == 'DSession')
+        is_slave = (hasattr(session.config, 'slaveinput') or
+                    session_name == 'SlaveSession')
+        nodeid = None
+
+        if is_master:
+            controller_cls = cov_core.DistMaster
+        elif is_slave:
+            controller_cls = cov_core.DistSlave
+            nodeid = session.config.slaveinput.get('slaveid', getattr(session, 'nodeid'))
+        else:
+            controller_cls = cov_core.Central
+
         self.cov_controller = controller_cls(cov_source,
                                              cov_report,
                                              cov_config,
                                              session.config,
-                                             getattr(session, 'nodeid', None))
+                                             nodeid)
 
         self.cov_controller.start()
 
@@ -301,10 +339,13 @@ class CovPlugin(object):
 
         self.cov_controller.summary(terminalreporter._tw)
 
-    def pytest_funcarg__cov(self, request):
-        """A pytest funcarg that provide access to the underlying coverage object."""
 
-        if self.cov_controller:
-            return self.cov_controller.cov
-        else:
-            return None
+def pytest_funcarg__cov(request):
+    """A pytest funcarg that provides access to the underlying coverage object."""
+
+    # Check with hasplugin to avoid getplugin exception in older pytest.
+    if request.config.pluginmanager.hasplugin('_cov'):
+        plugin = request.config.pluginmanager.getplugin('_cov')
+        if plugin.cov_controller:
+            return plugin.cov_controller.cov
+    return None
